#include <stdlib.h>

#include "bpipe/bpipe.h"

#include <testlib/testlib.h>

char buf[1024];

static char *
splat( BPMatrix *matrix )
{
  int ii;
  char tbuf[256];

  buf[0] = '\0';

  for ( ii = 0 ; ii < matrix->nd ; ii++ )
  {
    sprintf( tbuf, "[%d]", matrix->extent[ii] );
    strcat( buf, tbuf );
  }

  return buf;
}


int main( void )
{
  char label[1024];
  
  START( "squeeze and crunch" );

#define LABEL(extra) (sprintf( label, "%-10s: %s", extra, splat(matrix) ), label)

  /*******************************************************************/
  {
    BPMatrix *sq;
    BPMatrix *matrix = bpipe_matrix_new_va( 4, 1, 2, 3, 1 );

    sq = bpipe_matrix_squeeze( matrix );
    TESTSTRING( LABEL("squeeze"), splat(sq), "[1][2][3]" );
    bpipe_matrix_delete( sq );

    sq = bpipe_matrix_crunch( matrix );
    TESTSTRING( LABEL("crunch"), splat(sq), "[2][3]" );
    bpipe_matrix_delete( sq );

    bpipe_matrix_delete( matrix );
  }

  /*******************************************************************/
  {
    BPMatrix *sq;
    BPMatrix *matrix = bpipe_matrix_new_va( 4, 1, 1, 3, 1 );

    sq = bpipe_matrix_squeeze( matrix );
    TESTSTRING( LABEL("squeeze"), splat(sq), "[1][1][3]" );
    bpipe_matrix_delete( sq );

    sq = bpipe_matrix_crunch( matrix );
    TESTSTRING( LABEL("crunch"), splat(sq), "[3]" );
    bpipe_matrix_delete( sq );

    bpipe_matrix_delete( matrix );
  }

  /*******************************************************************/
  {
    BPMatrix *sq;
    BPMatrix *matrix = bpipe_matrix_new_va( 4, 1, 1, 1, 1 );

    sq = bpipe_matrix_squeeze( matrix );
    TESTSTRING( LABEL("squeeze"), splat(sq), "[1]" );
    bpipe_matrix_delete( sq );

    sq = bpipe_matrix_crunch( matrix );
    TESTSTRING( LABEL("crunch"), splat(sq), "[1]" );
    bpipe_matrix_delete( sq );

    bpipe_matrix_delete( matrix );
  }
  
  /*******************************************************************/
  {
    BPMatrix *sq;
    BPMatrix *matrix = bpipe_matrix_new_va( 1, 1 );

    sq = bpipe_matrix_squeeze( matrix );
    TESTSTRING( LABEL("squeeze"), splat(sq), "[1]" );
    bpipe_matrix_delete( sq );

    sq = bpipe_matrix_crunch( matrix );
    TESTSTRING( LABEL("crunch"), splat(sq), "[1]" );
    bpipe_matrix_delete( sq );

    bpipe_matrix_delete( matrix );
  }

  /*******************************************************************/
  SUMMARY();
  return FAILED( ) ? EXIT_FAILURE : EXIT_SUCCESS;


}
